@@ -33,7 +33,43 @@
                    "a"))
     ;; Non-nested
     (should (equal (ht-get* (ht (1 "one")) 1)
-                   "one"))))
+                   "one"))
+    ;; Base case (no keys)
+    (should (equal (ht-get* alphabets)
+                   alphabets))
+    ;; Works with apply, see #38
+    (should (equal (apply #'ht-get* (list (ht) 1))
+                   nil))))
+
+(ert-deftest ht-test-setf-ht-get ()
+  (let ((test-table (ht (1 "one"))))
+    (should (equal (setf (ht-get test-table 1) "alpha") "alpha")))
+
+  (let ((test-table (ht (1 "one") (2 "two"))))
+    (setf (ht-get test-table 1)  "alpha")
+    (should (equal (ht-get test-table 1)
+                   "alpha"))))
+
+(ert-deftest ht-test-setf-ht-get* ()
+  (let ((test-table (ht (1 (ht (2 (ht (3 "three"))))))))
+    (should (equal (setf (ht-get* test-table 1 2 3) "gamma")
+                   "gamma")))
+
+  ;; nested tables { 1 : { 2 : { 3 : three } } }
+  (let ((test-table (ht (1 (ht (2 (ht (3 "three"))))))))
+    (setf (ht-get* test-table 1 2 3) "gamma")
+    (should (equal (ht-get* test-table 1 2 3)
+                   "gamma")))
+  ;; nested tables { 1 : { 2 : two } }
+  (let ((test-table (ht (1 (ht (2 "two"))))))
+    (setf (ht-get* test-table 1 2) "beta")
+    (should (equal (ht-get* test-table 1 2)
+                   "beta")))
+  ;; Non-nested table { 1 : one }
+  (let ((test-table (ht (1 "one"))))
+    (setf (ht-get* test-table 1) "alpha")
+    (should (equal (ht-get* test-table 1)
+                   "alpha"))))
 
 (ert-deftest ht-test-update ()
   (let ((test-table (ht ("foo" 1))))
@@ -165,6 +201,11 @@
          (test-table (ht-from-plist plist)))
     (should (equal (ht-items test-table) '(("key1" "value1"))))))
 
+(ert-deftest ht-test-from-plist-masked-values ()
+  (let* ((plist '("key1" "value1" "key2" "value2" "key1" "value3"))
+         (test-table (ht-from-plist plist)))
+    (should (equal (ht-get test-table "key1") "value1"))))
+
 (ert-deftest ht-test-to-alist ()
   (let* ((alist '(("key1" . "value1") ("key2" . "value2")))
          (test-table (ht-from-alist alist)))
@@ -277,6 +318,26 @@
   (should (not (ht-equal-p (ht (1 2)) (ht (1 2) (3 4)))))
   (should (not (ht-equal-p (ht (1 2) (3 4)) (ht (1 2))))))
 
+(ert-deftest ht-test-two-name-style-predicator ()
+  (let ((real-definition (lambda (sym)
+                           (let ((def sym))
+                             (while (and def (symbolp def))
+                               (setq def (symbol-function def)))
+                             def))))
+    (mapatoms
+     (lambda (sym)
+       (let ((sym-name (symbol-name sym)))
+         (cond
+          ;; Ignore tests
+          ((string-match-p "\\`ht-test" sym-name)
+           nil)
+          ((string-match "\\`\\(ht.*\\)\\(?:-p\\|\\?\\)\\'" sym-name)
+           (let* ((name-body (match-string 1 sym-name))
+                  (scheme-style-sym (intern (format "%s?" name-body)))
+                  (cl-style-sym (intern (format "%s-p" name-body))))
+             (should (eq (funcall real-definition scheme-style-sym)
+                         (funcall real-definition cl-style-sym)))))))))))
+
 (defun ht-run-tests ()
   (interactive)
   (ert-run-tests-interactively "ht-test-"))
